<?php

namespace FiloBlu\Refilo\Model\Indexer;

use FiloBlu\Refilo\Helper\Data as HelperData;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Indexer\Processor\ContentType as ProcessorContentType;
use Magento\Framework\Mview\ActionInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class ContentType
 * @package FiloBlu\Refilo\Model\Indexer
 */
class ContentType implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var ProcessorContentType
     */
    private $processorContentType;
    /**
     * @var integer
     */
    private $batchSize;
    /**
     * @var integer
     */
    private $batchSequence;
    /**
     * @var Mongo
     */
    private $connection;
    /**
     * @var HelperData
     */
    private $helperData;

    /**
     * Catalog constructor.
     * @param StoreManagerInterface $storeManager
     * @param ProcessorContentType $processorContentType
     * @param Mongo $connection
     * @param HelperData $helperData
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ProcessorContentType $processorContentType,
        Mongo $connection,
        HelperData $helperData
    )
    {
        $this->connection = $connection;
        $this->helperData = $helperData;
        $this->_storeManager = $storeManager;
        $this->processorContentType = $processorContentType;
        $this->batchSequence = 1;
    }

    /*
     * Used by mview, allows process indexer in the "Update on schedule" mode
     */
    public function execute($ids)
    {
        $this->executeGeneric($ids);
    }

    /**
     * @param bool $ids
     */
    protected function executeGeneric($ids = false)
    {
        $this->batchSize = ($batch_size = (int)$this->helperData->getConfigValue("sync/data/batch_size")) ? $batch_size : 100;
        $storeDataList = $this->_storeManager->getStores();
        echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640

        foreach ($storeDataList as $key => $store) {
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $starttime = $mtime;
            printf("Prepare for store %s", $store->getCode() . "\n\r");
            while ($contentTypeItems = $this->processorContentType->getItems($store, $this->batchSize, $this->batchSequence, $ids)) {
                echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
                $mongo_collection = "cms.contents_" . $store->getCode();
                $this->connection->push("_id", $contentTypeItems, $mongo_collection);
                $this->batchSequence++;
            }
            $this->processorContentType->setTotalPages(0);
            $this->batchSequence = 1;
            //$this->connection->ensureIndex();
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $endtime = $mtime;
            $totaltime = ($endtime - $starttime);
            echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
            echo "Executed in " . $totaltime . " seconds";
        }
    }

    public function executeFull()
    {
        $this->executeGeneric();
    }

    /*
     * Works with a set of entity changed (may be massaction)
     */

    public function executeList(array $ids)
    {
        //Works with a set of placed orders (mass actions and so on)
        $this->executeGeneric($ids);
    }

    /*
     * Works in runtime for a single entity using plugins
     */
    public function executeRow($id)
    {
        //Works in runtime for a single order using plugins
        $this->executeGeneric([$id]);
    }
}
