<?php

namespace FiloBlu\Refilo\Model\Indexer;

use FiloBlu\Refilo\Helper\Data as HelperData;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Indexer\Processor\Pages as ProcessorPages;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Mview\ActionInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Pages
 * @package FiloBlu\Refilo\Model\Indexer
 */
class Pages implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{
    /**
     * @var ProcessorPages
     */
    private $processorPages;

    /**
     * @var integer
     */
    private $batchSize;
    /**
     * @var integer
     */
    private $batchSequence;
    /**
     * @var Mongo
     */
    private $connection;
    /**
     * @var HelperData
     */
    private $helperData;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * Catalog constructor.
     * @param StoreManagerInterface $storeManager
     * @param ProcessorPages $processorPages
     * @param Mongo $connection
     * @param HelperData $helperData
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ProcessorPages $processorPages,
        Mongo $connection,
        HelperData $helperData,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->connection = $connection;
        $this->helperData = $helperData;
        $this->_storeManager = $storeManager;
        $this->processorPages = $processorPages;
        $this->scopeConfig = $scopeConfig;

        $this->batchSequence = 1;
    }

    /*
     * Used by mview, allows process indexer in the "Update on schedule" mode
     */
    public function execute($ids)
    {
        $this->executeGeneric($ids);
    }

    /**
     * @param bool $ids
     */
    protected function executeGeneric($ids = false)
    {
        $this->batchSize = ($batch_size = (int)$this->helperData->getConfigValue("sync/data/batch_size")) ? $batch_size : 100;

        $storeDataList = $this->_storeManager->getStores();
        if (php_sapi_name() === 'cli') {
            echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
        }

        foreach ($storeDataList as $key => $store) {
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $starttime = $mtime;
            if (php_sapi_name() === 'cli') {
                printf("Prepare for store %s", $store->getCode() . "\n\r");
            }
            while ($blockItems = $this->processorPages->getItems($store, $this->batchSize, $this->batchSequence, $ids)) {
                if (php_sapi_name() === 'cli') {
                    echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
                }
                $mongo_collection = "contents_" . $store->getCode();
                $this->connection->update(
                    "_id",
                    $blockItems,
                    $mongo_collection,
                    true,
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );
                $this->batchSequence++;
            }
            $this->processorPages->setTotalPages(0);
            $this->batchSequence = 1;
//            $this->connection->ensureIndex();
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $endtime = $mtime;
            $totaltime = ($endtime - $starttime);
            if (php_sapi_name() === 'cli') {
                echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
                echo "Executed in " . $totaltime . " seconds";
            }
        }
    }

    public function executeFull()
    {
        $this->executeGeneric();
    }

    /*
     * Works with a set of entity changed (may be massaction)
     */

    public function executeList(array $ids)
    {
        //Works with a set of placed orders (mass actions and so on)
        $this->executeGeneric($ids);
    }

    /*
     * Works in runtime for a single entity using plugins
     */
    public function executeRow($id)
    {
        //Works in runtime for a single order using plugins
        $this->executeGeneric([$id]);
    }
}
