<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor;

use FiloBlu\Refilo\Model\Indexer\Processor\Blocks\Renderer;
use Magento\CatalogWidget\Model\RuleFactory;
use Magento\Cms\Model\ResourceModel\Block\CollectionFactory as MagentoBlocks;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Data\Wysiwyg\Normalizer as FilterNormalized;
use Magento\Framework\Filter\Template as FilterTemplate;
use Magento\Framework\Filter\Template\Tokenizer\Parameter;
use Magento\Framework\Serialize\Serializer\Json as SerializerJson;

class Blocks
{
    protected $magentoBlocks;

    /**
     * @var int
     */
    protected $totalPages = 0;

    /**
     * @var ResourceConnection
     */
    protected $connection;
    /**
     * @var SerializerJson
     */
    protected $serializerJson;
    /**
     * @var RuleFactory
     */
    protected $rule;
    /**
     * @var FilterTemplate
     */
    protected $filterTemplate;
    /**
     * @var FilterNormalized
     */
    protected $filterNormalizer;
    /**
     * @var FiloBlu\Refilo\Model\Indexer\Processor\Blocks\Renderer
     */
    protected $blockRender;

    /**
     * Catalog constructor.
     * @param ResourceConnection $connection
     * @param MagentoBlocks $magentoBlocks
     * @param FilterTemplate $filterTemplate
     * @param SerializerJson $serializerJson
     * @param RuleFactory $ruleFactory
     * @param FilterNormalized $filterNormalizer
     * @param FiloBlu\Refilo\Model\Indexer\Processor\Blocks\Renderer $blockRender
     */
    public function __construct(
        ResourceConnection $connection,
        MagentoBlocks $magentoBlocks,
        FilterTemplate $filterTemplate,
        SerializerJson $serializerJson,
        RuleFactory $ruleFactory,
        FilterNormalized $filterNormalizer,
        Renderer $blockRender
    ) {
        $this->connection = $connection->getConnection();
        $this->magentoBlocks = $magentoBlocks;
        $this->serializerJson = $serializerJson;
        $this->rule = $ruleFactory->create();
        $this->filterTemplate = $filterTemplate;
        $this->filterNormalizer = $filterNormalizer;
        $this->blockRender = $blockRender;
    }

    public function getItems($store, $batch_size, $batch_sequence, $ids = false)
    {
        $contentTypeCollection = $this->prepareItemsCollection($store, $batch_size, $batch_sequence, $ids);
        return ($contentTypeCollection) ? $this->processItemsCollection($contentTypeCollection, $store) : false;
    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return \Blackbird\ContentManager\Model\ResourceModel\Content\Collection|bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function prepareItemsCollection($store, $batch_size, $batch_sequence, $ids = false)
    {
        if (($this->totalPages < $batch_sequence) && $this->totalPages != 0) {
            return false;
        }

        $collection = $this->magentoBlocks->create();

        $collection->addStoreFilter($store);

        if (is_array($ids)) {
            $collection->addFilter('entity_id', $ids, 'in');
        }

        $collection->setPageSize($batch_size);
        $collection->setCurPage($batch_sequence);
        if (php_sapi_name() === 'cli') {
            printf("Prepare Collection\n\r");
        }

        if (!$this->totalPages) {
            $count = $collection->count();
            $this->totalPages = ceil($count / $batch_size);
        }
        if (php_sapi_name() === 'cli') {
            printf("Page %d of %d \n\r", $batch_sequence, $this->totalPages);
        }
        return $collection;
    }

    /**
     * @param $_blocks_collection
     * @return array
     */
    protected function processItemsCollection($_blocks_collection, $store)
    {
        $items = [];

        foreach ($_blocks_collection->getItems() as $item) {
            $block = new \stdClass();

            $block->_id = "block_" . $item->getId() . "_" . $store->getData('code');
            $block->metacode = "block";
            $block->store = $store->getData('code');
            $block->content_id = (int)  $item->getId();
            $block->url_key = $item->getIdentifier();
            $block->idx = $item->getIdentifier();
            $block->title = $item->getTitle();
            $block->type = "static_block";
            $block->status = $item->isActive();
            $block->content = $item->getContent();
            $block->display_entity = $item->getDisplayOn();
            $block->display_selection_type = $item->getSelectionType();
            $block->display_selection_target = $item->getSelectionTarget();
            $block->display_container = $item->getRefiloLayoutContainer();
            $block->display_sort_order = $item->getRefiloLayoutOrder();
            $block->metadata = new \stdClass();
            $block->metacontent = new \stdClass();

            $items[] = $block;
        }

        return $items;
    }

    /**
     * @param $content
     * @return mixed
     */
    public function processContent(&$content)
    {
        $metacontents = [];
        // "depend", "if", and "template" directives should be first
        foreach ([
                     $this->filterTemplate::CONSTRUCTION_DEPEND_PATTERN => 'dependDirective',
                     $this->filterTemplate::CONSTRUCTION_IF_PATTERN => 'ifDirective',
                     $this->filterTemplate::CONSTRUCTION_TEMPLATE_PATTERN => 'templateDirective',
                     $this->filterTemplate::CONSTRUCTION_PATTERN => 'Directive',
                     $this->filterTemplate::LOOP_PATTERN => 'Loop'
                 ] as $pattern => $directive) {
            if (preg_match_all($pattern, $content, $constructions, PREG_SET_ORDER)) {
                switch ($directive) {
                    case "Directive":
                        $metacontents = array_merge($this->processDirective($constructions, $content));
                        break;
                }
            }
        }

        $targetcontents = [];
        foreach ($metacontents as $key=>$metacontent) {
            if ($metacontent["target"]) {
                $idx = $key + 1;
                $tag = '<metacontent id = "' . "metacontent_{$idx}" . '" />';
                $targetcontents["metacontent_{$idx}"] = $metacontent["target"];
                $content = str_replace($metacontent["original"], $tag, $content);
            } else {
                $content = str_replace($metacontent["original"], "", $content);
            }
        }

        return $targetcontents;
    }

    /**
     * @param $constructions
     * @param $content
     * @return mixed
     */
    protected function processDirective($constructions, &$content)
    {
        $metacontents = [];

        foreach ($constructions as $key => $construction) {
            $class = $construction[1];
            $tokenizer = new Parameter();
            $tokenizer->setString($construction[2]);

            $block_params = $tokenizer->tokenize();

            switch ($class) {

                case "block":
                    $metacontent = $this->blockRender->renderBlock($block_params);
                    $metacontents[] = ["original"=>$construction[0],"target"=>$metacontent];
                    break;
                case "widget":
                    $metacontent = $this->blockRender->renderWidget($block_params);
                    $metacontents[] = ["original"=>$construction[0],"target"=>$metacontent];
                    break;
                default:
                    break;
            }
            // $tag = '<metacontent id = "' . $key . '" />';

//            $content = str_replace($construction[0], $tag, $content);
        }

        return $metacontents;
    }

    /**
     * @return int
     */
    public function getTotalPages()
    {
        return $this->totalPages;
    }

    /**
     * @param int $total_pages
     */
    public function setTotalPages($totalPages)
    {
        $this->totalPages = $totalPages;
    }
}
